@@ -713,3 +713,13 @@ RM_STATUS NV_API_CALL os_get_address_space_info(
 {
     return RM_ERR_NOT_SUPPORTED;
 }
+
+RM_STATUS NV_API_CALL os_alloc_smp_barrier(void)
+{
+    return RM_OK;
+}
+
+RM_STATUS NV_API_CALL os_free_smp_barrier(void)
+{
+    return RM_OK;
+}
