@@ -1112,10 +1112,11 @@ NvS32 NV_API_CALL nv_free_kernel_mapping(
 
 NvS32 nv_alloc_contig_pages(
     nv_state_t *nv,
-    NvU32 count,
-    NvU32 cache_type,
-    NvU64 *pte_array,
-    void **private
+    NvU32       count,
+    NvU32       cache_type,
+    NvBool      zero,
+    NvU64      *pte_array,
+    void      **private
 )
 {
     struct nvidia_alloc *at;
@@ -1123,6 +1124,7 @@ NvS32 nv_alloc_contig_pages(
     vm_memattr_t attr;
     vm_offset_t address;
     NvU32 size = (count * PAGE_SIZE);
+    int flags = (zero ? M_ZERO : 0);
     int status;
 
     switch (cache_type) {
@@ -1160,8 +1162,8 @@ NvS32 nv_alloc_contig_pages(
         return ENOMEM;
     }
 
-    address = kmem_alloc_contig(kernel_map, size, 0, 0, sc->dma_mask,
-            PAGE_SIZE, 0, attr);
+    address = kmem_alloc_contig(kernel_map, size, flags, 0,
+            sc->dma_mask, PAGE_SIZE, 0, attr);
     if (!address) {
         status = ENOMEM;
         goto failed;
@@ -1248,10 +1250,11 @@ NvS32 nv_free_contig_pages(
 
 NvS32 nv_alloc_system_pages(
     nv_state_t  *nv,
-    NvU32 count,
-    NvU32 cache_type,
-    NvU64 *pte_array,
-    void **private
+    NvU32        count,
+    NvU32        cache_type,
+    NvBool       zero,
+    NvU64       *pte_array,
+    void       **private
 )
 {
     struct nvidia_alloc *at;
@@ -1260,6 +1263,7 @@ NvS32 nv_alloc_system_pages(
     uint32_t i;
     vm_memattr_t attr;
     uint32_t size = (count * PAGE_SIZE);
+    int flags = (zero ? M_ZERO : 0);
     int status;
 
     switch (cache_type) {
@@ -1298,8 +1302,8 @@ NvS32 nv_alloc_system_pages(
     }
 
     for (i = 0; i < count; i++) {
-        address = kmem_alloc_contig(kernel_map, PAGE_SIZE, 0, 0, sc->dma_mask,
-                PAGE_SIZE, 0, attr);
+        address = kmem_alloc_contig(kernel_map, PAGE_SIZE, flags, 0,
+                sc->dma_mask, PAGE_SIZE, 0, attr);
         if (!address) {
             status = ENOMEM;
             goto failed;
@@ -1464,12 +1468,12 @@ NvS32 nv_free_agp_pages(
 
 RM_STATUS NV_API_CALL nv_alias_pages(
     nv_state_t *nv,
-    NvU32 page_cnt,
-    NvU32 contiguous,
-    NvU32 cache_type,
-    NvU64 guest_id,
-    NvU64 *pte_array,
-    void **priv_data
+    NvU32       count,
+    NvU32       alloc_type_contiguous,
+    NvU32       cache_type,
+    NvU64       guest_id,
+    NvU64      *pte_array,
+    void      **priv_data
 )
 {
     return RM_ERR_NOT_SUPPORTED;
@@ -1488,17 +1492,19 @@ RM_STATUS NV_API_CALL nv_guest_pfn_list(
 
 RM_STATUS NV_API_CALL nv_alloc_pages(
     nv_state_t *nv,
-    NvU32 count,
-    NvU32 alloc_type_agp,
-    NvU32 alloc_type_contiguous,
-    NvU32 cache_type,
-    NvU64 *pte_array,
-    void **private
+    NvU32       count,
+    NvBool      alloc_type_agp,
+    NvBool      alloc_type_contiguous,
+    NvU32       cache_type,
+    NvBool      alloc_type_zeroed,
+    NvU64      *pte_array,
+    void      **private
 )
 {
     NvU32 offset;
     RM_STATUS status = RM_ERR_NO_FREE_MEM;
     nv_stack_t *sp = NULL;
+    NvBool zero = alloc_type_zeroed;
 
     if (alloc_type_agp) {
         if (!NV_AGP_ENABLED(nv))
@@ -1539,11 +1545,15 @@ RM_STATUS NV_API_CALL nv_alloc_pages(
         }
     } else {
         if (!alloc_type_contiguous) {
-            if (nv_alloc_system_pages(nv, count, cache_type, pte_array, private))
+            if (nv_alloc_system_pages(nv, count, cache_type, zero,
+                        pte_array, private)) {
                 goto failed;
+            }
         } else {
-            if (nv_alloc_contig_pages(nv, count, cache_type, pte_array, private))
+            if (nv_alloc_contig_pages(nv, count, cache_type, zero,
+                        pte_array, private)) {
                 goto failed;
+            }
         }
         return RM_OK;
     }
@@ -1857,7 +1867,47 @@ RM_STATUS NV_API_CALL nv_acpi_ddc_method(
     return RM_ERR_NOT_SUPPORTED;
 }
 
+RM_STATUS NV_API_CALL nv_acpi_rom_method(
+    nv_state_t *nv,
+    NvU32 *pInData,
+    NvU32 *pOutData
+)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
+
 void* NV_API_CALL nv_get_smu_state(void)
 {
     return NULL;
 }
+
+RM_STATUS NV_API_CALL nv_lock_user_pages(
+    nv_state_t *nv,
+    void       *address,
+    NvU64       count,
+    NvU64      *pte_array,
+    void      **private
+)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
+
+RM_STATUS NV_API_CALL nv_unlock_user_pages(
+    nv_state_t *nv,
+    NvU64       count,
+    NvU64      *pte_array,
+    void       *private
+)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
+
+RM_STATUS NV_API_CALL nv_log_error(
+    nv_state_t *nv,
+    NvU32       error_number,
+    const char *format,
+    void       *ap
+)
+{
+    return RM_OK;
+}
