@@ -248,17 +248,20 @@ static void* nvidia_find_bridge(void)
 int nvidia_sysctl_dev_model(SYSCTL_HANDLER_ARGS)
 {
     nv_state_t *nv = arg1;
+    device_t dev = nv->handle;
     nv_stack_t *sp;
-
     char model_name[NV_DEVICE_NAME_LENGTH+1];
-    NvU16 id = nv->device_id;
+    NvU16 subvendor, subdevice;
 
     NV_UMA_ZONE_ALLOC_STACK(sp);
     if (sp == NULL)
         return ENOMEM;
 
-    if (rm_get_device_name(sp, nv, id, NV_DEVICE_NAME_LENGTH, model_name)
-            != RM_OK) {
+    subvendor = pci_get_subvendor(dev);
+    subdevice = pci_get_subdevice(dev);
+
+    if (rm_get_device_name(sp, nv, nv->device_id, subvendor, subdevice,
+                NV_DEVICE_NAME_LENGTH, model_name) != RM_OK) {
         strcpy(model_name, "Unknown");
     }
 
