@@ -104,6 +104,8 @@ void        NV_API_CALL  os_dbg_init                 (void);
 void        NV_API_CALL  os_dbg_breakpoint           (void);
 void        NV_API_CALL  os_dbg_set_level            (NvU32);
 NvU32       NV_API_CALL  os_get_cpu_count            (void);
+RM_STATUS   NV_API_CALL  os_alloc_smp_barrier        (void);
+RM_STATUS   NV_API_CALL  os_free_smp_barrier         (void);
 RM_STATUS   NV_API_CALL  os_raise_smp_barrier        (void);
 RM_STATUS   NV_API_CALL  os_clear_smp_barrier        (void);
 void        NV_API_CALL  os_disable_preemption       (void);
